@@ -29,7 +29,6 @@
 #else
 # import <Cocoa/Cocoa.h>
 # import <ScreenSaver/ScreenSaver.h>
-//# define USE_TOUCHBAR
 #endif
 
 
@@ -38,7 +37,7 @@
 
 #ifdef HAVE_IPHONE
 
-# if defined(TARGET_OS_TV) && !defined(HAVE_TVOS)
+# if TARGET_OS_TV && !defined(HAVE_TVOS)
    // We aren't receiving this from Xcode when compiling libjwxyz.a for tvOS.
 #  define HAVE_TVOS 1
 # endif
@@ -76,8 +75,6 @@
 			      <UIApplicationDelegate>
 # elif defined(HAVE_IPHONE)
 			      <UIAlertViewDelegate>
-# elif defined(USE_TOUCHBAR)
-			      <NSTouchBarDelegate>
 # endif
 {
   struct xscreensaver_function_table *xsft;
@@ -120,11 +117,6 @@
 
 # endif // !HAVE_IPHONE
 
-# ifdef USE_TOUCHBAR
-  XScreenSaverView *touchbar_view;
-  BOOL touchbar_p;
-# endif
-
   NSOpenGLContext *ogl_ctx;      // OpenGL rendering context
 
 # ifdef JWXYZ_QUARTZ
