@@ -1,5 +1,5 @@
 /* demo-Gtk-conf.c --- implements the dynamic configuration dialogs.
- * xscreensaver, Copyright (c) 2001, 2002 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright © 2001-2022 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -19,10 +19,11 @@ typedef struct {
   char *progname;
   char *progclass;
   char *description;
+  int year;
 } conf_data;
 
 extern conf_data *load_configurator (const char *cmd_line, gboolean verbose_p);
-extern char *get_configurator_command_line (conf_data *);
+extern char *get_configurator_command_line (conf_data *, gboolean default_p);
 extern void  set_configurator_command_line (conf_data *, const char *cmd_line);
 extern void free_conf_data (conf_data *);
 
