@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# Copyright © 2008-2021 Jamie Zawinski <jwz@jwz.org>
+# Copyright © 2008-2022 Jamie Zawinski <jwz@jwz.org>
 #
 # Permission to use, copy, modify, distribute, and sell this software and its
 # documentation for any purpose is hereby granted without fee, provided that
@@ -21,7 +21,7 @@ use diagnostics;
 use strict;
 
 my $progname = $0; $progname =~ s@.*/@@g;
-my ($version) = ('$Revision: 1.38 $' =~ m/\s(\d[.\d]+)\s/s);
+my ($version) = ('$Revision: 1.39 $' =~ m/\s(\d[.\d]+)\s/s);
 
 my $verbose = 0;
 my $debug_p = 0;
@@ -784,8 +784,7 @@ sub build_android(@) {
       my $rsrc  = $widget->{resource};
       my $label = $widget->{_label};
       my $def   = $widget->{default};
-      my $invert_p = (($widget->{convert} || '') eq 'invert');
-
+      my $cvt   = $widget->{convert} || '';
       my $key   = "${saver}_$rsrc" if $rsrc;
 
       #### The menus don't actually have titles on X11 or Cocoa...
@@ -809,7 +808,9 @@ sub build_android(@) {
         $high_label = $high unless defined($high_label);
 
         ($low, $high) = ($high, $low)
-          if (($widget->{convert} || '') eq 'invert');
+          if ($cvt eq 'invert');
+
+        # I guess there's no way to implement convert="ratio" for Android.
 
         $settings .=
           ("<$package.SliderPreference\n" .
@@ -825,7 +826,7 @@ sub build_android(@) {
 
       } elsif ($type eq 'boolean') {
 
-        my $def = ($invert_p ? 'true' : 'false');
+        my $def = ($cvt eq 'invert' ? 'true' : 'false');
         $settings .=
           ("<CheckBoxPreference\n" .
            "  android:key=\"${key}\"\n" .
